#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "dcrf32.h"
#include "zbcr.h"
#include "errcode.h"
MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    //ui->pushButton->setVisible(false);
    ui->pushButton_3->setVisible(false);
}

MainWindow::~MainWindow()
{
    delete ui;
}



static const unsigned char *ConvertString(const unsigned char *in_str, unsigned char *out_str) {
#ifdef WIN32
  return in_str;
#else
  dc_string_converter(2, in_str, out_str);
  return out_str;
#endif
}

static int IdCardTest(HANDLE handle,QTextBrowser *tb) {
  int result;
  unsigned char info_buffer[64];
  int text_len;
  unsigned char text[256];
  int photo_len;
  unsigned char photo[1024];
  int fingerprint_len;
  unsigned char fingerprint[1024];
  int extra_len;
  unsigned char extra[70];
  int type;
#ifndef WIN32
  unsigned char title[128];
#endif

  printf("\n-------- ID Card Test --------\n");
tb->append("-------- ID Card Test --------");
  printf("dc_SamAReadCardInfo ... ");
  result = dc_SamAReadCardInfo(handle, 3, &text_len, text, &photo_len, photo, &fingerprint_len, fingerprint, &extra_len, extra);
  if (result != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  type = 0;
  if ((text[248] == 'I') && (text[249] == 0)) {
    type = 1;
  } else if ((text[248] == 'J') && (text[249] == 0)) {
    type = 2;
  }

  if (type == 0) {
    unsigned char name[64];
    unsigned char sex[8];
    unsigned char nation[12];
    unsigned char birth_day[36];
    unsigned char address[144];
    unsigned char id_number[76];
    unsigned char department[64];
    unsigned char expire_start_day[36];
    unsigned char expire_end_day[36];
    unsigned char reserved[76];

    printf("dc_ParseTextInfo ... ");
#ifdef WIN32
    result = dc_ParseTextInfo(handle, 1, text_len, text, name, sex, nation, birth_day, address, id_number, department, expire_start_day, expire_end_day, reserved);
#else
    result = dc_ParseTextInfo(handle, 2, text_len, text, name, sex, nation, birth_day, address, id_number, department, expire_start_day, expire_end_day, reserved);
#endif
    if (result != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");
#ifdef WIN32
    wprintf(L"%s: %s\n", L"姓名", (wchar_t *)name);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"姓名", title), name);
#endif
    //ui.textBrowser->
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x10, sex, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x20, sex, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s\n", L"性别", (wchar_t *)info_buffer);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"性别", title), info_buffer);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x11, nation, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x21, nation, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s\n", L"民族", (wchar_t *)info_buffer);
    wprintf(L"%s: %s\n", L"出生日期", (wchar_t *)birth_day);
    wprintf(L"%s: %s\n", L"住址", (wchar_t *)address);
    wprintf(L"%s: %s\n", L"公民身份号码", (wchar_t *)id_number);
    wprintf(L"%s: %s\n", L"签发机关", (wchar_t *)department);
    wprintf(L"%s: %s\n", L"证件签发日期", (wchar_t *)expire_start_day);
    wprintf(L"%s: %s\n", L"证件终止日期", (wchar_t *)expire_end_day);
#else
    QString qstr;// = QString("%1").arg(num);
    printf("%s: %s\n", ConvertString((unsigned char *)"民族", title), info_buffer);
    tb->append("民族");
    qstr = QString::fromUtf8((char *)info_buffer);
    tb->append(qstr);
    printf("%s: %s\n", ConvertString((unsigned char *)"出生日期", title), birth_day);

    tb->append("出生日期");
    qstr = QString::fromUtf8((char *)birth_day);
    tb->append(qstr);

    printf("%s: %s\n", ConvertString((unsigned char *)"住址", title), address);

    tb->append("住址");
    qstr = QString::fromUtf8((char *)address);
    tb->append(qstr);
    printf("%s: %s\n", ConvertString((unsigned char *)"公民身份号码", title), id_number);

    tb->append("公民身份号码");
    qstr = QString::fromUtf8((char *)id_number);
    tb->append(qstr);

    printf("%s: %s\n", ConvertString((unsigned char *)"签发机关", title), department);

    tb->append("签发机关");
    qstr = QString::fromUtf8((char *)department);
    tb->append(qstr);

    printf("%s: %s\n", ConvertString((unsigned char *)"证件签发日期", title), expire_start_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件终止日期", title), expire_end_day);
#endif
  } else if (type == 1) {
    unsigned char english_name[244];
    unsigned char sex[8];
    unsigned char id_number[64];
    unsigned char citizenship[16];
    unsigned char chinese_name[64];
    unsigned char expire_start_day[36];
    unsigned char expire_end_day[36];
    unsigned char birth_day[36];
    unsigned char version_number[12];
    unsigned char department_code[20];
    unsigned char type_sign[8];
    unsigned char reserved[16];

    printf("dc_ParseTextInfoForForeigner ... ");
#ifdef WIN32
    result = dc_ParseTextInfoForForeigner(handle, 1, text_len, text, english_name, sex, id_number, citizenship, chinese_name, expire_start_day, expire_end_day, birth_day, version_number, department_code, type_sign, reserved);
#else
    result = dc_ParseTextInfoForForeigner(handle, 2, text_len, text, english_name, sex, id_number, citizenship, chinese_name, expire_start_day, expire_end_day, birth_day, version_number, department_code, type_sign, reserved);
#endif
    if (result != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");
#ifdef WIN32
    wprintf(L"%s: %s\n", L"英文姓名", (wchar_t *)english_name);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"英文姓名", title), english_name);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x10, sex, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x20, sex, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s/%s\n", L"性别", (wchar_t *)info_buffer, (wcscmp((wchar_t *)sex, L"1") == 0) ? L"M" : L"F");
    wprintf(L"%s: %s\n", L"永久居留证号码", (wchar_t *)id_number);
#else
    printf("%s: %s/%s\n", ConvertString((unsigned char *)"性别", title), info_buffer, (strcmp((char *)sex, "1") == 0) ? "M" : "F");
    printf("%s: %s\n", ConvertString((unsigned char *)"永久居留证号码", title), id_number);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x12, citizenship, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x22, citizenship, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s/%s\n", L"国籍或所在地区代码", (wchar_t *)info_buffer, (wchar_t *)citizenship);
    wprintf(L"%s: %s\n", L"中文姓名", (wchar_t *)chinese_name);
    wprintf(L"%s: %s\n", L"证件签发日期", (wchar_t *)expire_start_day);
    wprintf(L"%s: %s\n", L"证件终止日期", (wchar_t *)expire_end_day);
    wprintf(L"%s: %s\n", L"出生日期", (wchar_t *)birth_day);
    wprintf(L"%s: %s\n", L"证件版本号", (wchar_t *)version_number);
    wprintf(L"%s: %s\n", L"当次申请受理机关代码", (wchar_t *)department_code);
    wprintf(L"%s: %s\n", L"证件类型标识", (wchar_t *)type_sign);
#else
    printf("%s: %s/%s\n", ConvertString((unsigned char *)"国籍或所在地区代码", title), info_buffer, citizenship);
    printf("%s: %s\n", ConvertString((unsigned char *)"中文姓名", title), chinese_name);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件签发日期", title), expire_start_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件终止日期", title), expire_end_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"出生日期", title), birth_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件版本号", title), version_number);
    printf("%s: %s\n", ConvertString((unsigned char *)"当次申请受理机关代码", title), department_code);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件类型标识", title), type_sign);
#endif
  } else if (type == 2) {
    unsigned char name[64];
    unsigned char sex[8];
    unsigned char reserved[12];
    unsigned char birth_day[36];
    unsigned char address[144];
    unsigned char id_number[76];
    unsigned char department[64];
    unsigned char expire_start_day[36];
    unsigned char expire_end_day[36];
    unsigned char pass_number[40];
    unsigned char sign_count[12];
    unsigned char reserved2[16];
    unsigned char type_sign[8];
    unsigned char reserved3[16];

    printf("dc_ParseTextInfoForHkMoTw ... ");
#ifdef WIN32
    result = dc_ParseTextInfoForHkMoTw(handle, 1, text_len, text, name, sex, reserved, birth_day, address, id_number, department, expire_start_day, expire_end_day, pass_number, sign_count, reserved2, type_sign, reserved3);
#else
    result = dc_ParseTextInfoForHkMoTw(handle, 2, text_len, text, name, sex, reserved, birth_day, address, id_number, department, expire_start_day, expire_end_day, pass_number, sign_count, reserved2, type_sign, reserved3);
#endif
    if (result != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");
#ifdef WIN32
    wprintf(L"%s: %s\n", L"姓名", (wchar_t *)name);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"姓名", title), name);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x10, sex, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x20, sex, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s\n", L"性别", (wchar_t *)info_buffer);
    wprintf(L"%s: %s\n", L"出生日期", (wchar_t *)birth_day);
    wprintf(L"%s: %s\n", L"住址", (wchar_t *)address);
    wprintf(L"%s: %s\n", L"公民身份号码", (wchar_t *)id_number);
    wprintf(L"%s: %s\n", L"签发机关", (wchar_t *)department);
    wprintf(L"%s: %s\n", L"证件签发日期", (wchar_t *)expire_start_day);
    wprintf(L"%s: %s\n", L"证件终止日期", (wchar_t *)expire_end_day);
    wprintf(L"%s: %s\n", L"通行证号码", (wchar_t *)pass_number);
    wprintf(L"%s: %s\n", L"签发次数", (wchar_t *)sign_count);
    wprintf(L"%s: %s\n", L"证件类型标识", (wchar_t *)type_sign);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"性别", title), info_buffer);
    printf("%s: %s\n", ConvertString((unsigned char *)"出生日期", title), birth_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"住址", title), address);
    printf("%s: %s\n", ConvertString((unsigned char *)"公民身份号码", title), id_number);
    printf("%s: %s\n", ConvertString((unsigned char *)"签发机关", title), department);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件签发日期", title), expire_start_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件终止日期", title), expire_end_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"通行证号码", title), pass_number);
    printf("%s: %s\n", ConvertString((unsigned char *)"签发次数", title), sign_count);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件类型标识", title), type_sign);
#endif
  }

  printf("dc_ParsePhotoInfo ... ");
  result = dc_ParsePhotoInfo(handle, 0, photo_len, photo, 0, (unsigned char *)"me.bmp");
  if (result != 0) {
    if (result == -2) {
      printf("failed to call photo decoding library!\n");
    } else {
      printf("error!\n");
    }
    return -1;
  }
  printf("ok!\n");

  return 0;
}

static int IdCardTest2(HANDLE handle) {
  int result;
  unsigned char info_buffer[64];
  int text_len;
  unsigned char text[256];
  int photo_len;
  unsigned char photo[1024];
  int fingerprint_len;
  unsigned char fingerprint[1024];
  int extra_len;
  unsigned char extra[70];
  int type;
#ifndef WIN32
  unsigned char title[128];
#endif

  printf("\n-------- ID Card Test (Hardware Acceleration) --------\n");

  printf("dc_IdCardReadCardInfo ... ");
  result = dc_IdCardReadCardInfo(handle, 0x02, 0x0F, &text_len, text, &photo_len, photo, &fingerprint_len, fingerprint, &extra_len, extra);
  if (result != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  type = 0;
  if ((text[248] == 'I') && (text[249] == 0)) {
    type = 1;
  } else if ((text[248] == 'J') && (text[249] == 0)) {
    type = 2;
  }

  if (type == 0) {
    unsigned char name[64];
    unsigned char sex[8];
    unsigned char nation[12];
    unsigned char birth_day[36];
    unsigned char address[144];
    unsigned char id_number[76];
    unsigned char department[64];
    unsigned char expire_start_day[36];
    unsigned char expire_end_day[36];
    unsigned char reserved[76];

    printf("dc_ParseTextInfo ... ");
#ifdef WIN32
    result = dc_ParseTextInfo(handle, 1, text_len, text, name, sex, nation, birth_day, address, id_number, department, expire_start_day, expire_end_day, reserved);
#else
    result = dc_ParseTextInfo(handle, 2, text_len, text, name, sex, nation, birth_day, address, id_number, department, expire_start_day, expire_end_day, reserved);
#endif
    if (result != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");
#ifdef WIN32
    wprintf(L"%s: %s\n", L"姓名", (wchar_t *)name);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"姓名", title), name);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x10, sex, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x20, sex, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s\n", L"性别", (wchar_t *)info_buffer);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"性别", title), info_buffer);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x11, nation, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x21, nation, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s\n", L"民族", (wchar_t *)info_buffer);
    wprintf(L"%s: %s\n", L"出生日期", (wchar_t *)birth_day);
    wprintf(L"%s: %s\n", L"住址", (wchar_t *)address);
    wprintf(L"%s: %s\n", L"公民身份号码", (wchar_t *)id_number);
    wprintf(L"%s: %s\n", L"签发机关", (wchar_t *)department);
    wprintf(L"%s: %s\n", L"证件签发日期", (wchar_t *)expire_start_day);
    wprintf(L"%s: %s\n", L"证件终止日期", (wchar_t *)expire_end_day);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"民族", title), info_buffer);
    printf("%s: %s\n", ConvertString((unsigned char *)"出生日期", title), birth_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"住址", title), address);
    printf("%s: %s\n", ConvertString((unsigned char *)"公民身份号码", title), id_number);
    printf("%s: %s\n", ConvertString((unsigned char *)"签发机关", title), department);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件签发日期", title), expire_start_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件终止日期", title), expire_end_day);
#endif
  } else if (type == 1) {
    unsigned char english_name[244];
    unsigned char sex[8];
    unsigned char id_number[64];
    unsigned char citizenship[16];
    unsigned char chinese_name[64];
    unsigned char expire_start_day[36];
    unsigned char expire_end_day[36];
    unsigned char birth_day[36];
    unsigned char version_number[12];
    unsigned char department_code[20];
    unsigned char type_sign[8];
    unsigned char reserved[16];

    printf("dc_ParseTextInfoForForeigner ... ");
#ifdef WIN32
    result = dc_ParseTextInfoForForeigner(handle, 1, text_len, text, english_name, sex, id_number, citizenship, chinese_name, expire_start_day, expire_end_day, birth_day, version_number, department_code, type_sign, reserved);
#else
    result = dc_ParseTextInfoForForeigner(handle, 2, text_len, text, english_name, sex, id_number, citizenship, chinese_name, expire_start_day, expire_end_day, birth_day, version_number, department_code, type_sign, reserved);
#endif
    if (result != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");
#ifdef WIN32
    wprintf(L"%s: %s\n", L"英文姓名", (wchar_t *)english_name);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"英文姓名", title), english_name);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x10, sex, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x20, sex, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s/%s\n", L"性别", (wchar_t *)info_buffer, (wcscmp((wchar_t *)sex, L"1") == 0) ? L"M" : L"F");
    wprintf(L"%s: %s\n", L"永久居留证号码", (wchar_t *)id_number);
#else
    printf("%s: %s/%s\n", ConvertString((unsigned char *)"性别", title), info_buffer, (strcmp((char *)sex, "1") == 0) ? "M" : "F");
    printf("%s: %s\n", ConvertString((unsigned char *)"永久居留证号码", title), id_number);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x12, citizenship, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x22, citizenship, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s/%s\n", L"国籍或所在地区代码", (wchar_t *)info_buffer, (wchar_t *)citizenship);
    wprintf(L"%s: %s\n", L"中文姓名", (wchar_t *)chinese_name);
    wprintf(L"%s: %s\n", L"证件签发日期", (wchar_t *)expire_start_day);
    wprintf(L"%s: %s\n", L"证件终止日期", (wchar_t *)expire_end_day);
    wprintf(L"%s: %s\n", L"出生日期", (wchar_t *)birth_day);
    wprintf(L"%s: %s\n", L"证件版本号", (wchar_t *)version_number);
    wprintf(L"%s: %s\n", L"当次申请受理机关代码", (wchar_t *)department_code);
    wprintf(L"%s: %s\n", L"证件类型标识", (wchar_t *)type_sign);
#else
    printf("%s: %s/%s\n", ConvertString((unsigned char *)"国籍或所在地区代码", title), info_buffer, citizenship);
    printf("%s: %s\n", ConvertString((unsigned char *)"中文姓名", title), chinese_name);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件签发日期", title), expire_start_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件终止日期", title), expire_end_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"出生日期", title), birth_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件版本号", title), version_number);
    printf("%s: %s\n", ConvertString((unsigned char *)"当次申请受理机关代码", title), department_code);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件类型标识", title), type_sign);
#endif
  } else if (type == 2) {
    unsigned char name[64];
    unsigned char sex[8];
    unsigned char reserved[12];
    unsigned char birth_day[36];
    unsigned char address[144];
    unsigned char id_number[76];
    unsigned char department[64];
    unsigned char expire_start_day[36];
    unsigned char expire_end_day[36];
    unsigned char pass_number[40];
    unsigned char sign_count[12];
    unsigned char reserved2[16];
    unsigned char type_sign[8];
    unsigned char reserved3[16];

    printf("dc_ParseTextInfoForHkMoTw ... ");
#ifdef WIN32
    result = dc_ParseTextInfoForHkMoTw(handle, 1, text_len, text, name, sex, reserved, birth_day, address, id_number, department, expire_start_day, expire_end_day, pass_number, sign_count, reserved2, type_sign, reserved3);
#else
    result = dc_ParseTextInfoForHkMoTw(handle, 2, text_len, text, name, sex, reserved, birth_day, address, id_number, department, expire_start_day, expire_end_day, pass_number, sign_count, reserved2, type_sign, reserved3);
#endif
    if (result != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");
#ifdef WIN32
    wprintf(L"%s: %s\n", L"姓名", (wchar_t *)name);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"姓名", title), name);
#endif
#ifdef WIN32
    dc_ParseOtherInfo(handle, 0x10, sex, info_buffer);
#else
    dc_ParseOtherInfo(handle, 0x20, sex, info_buffer);
#endif
#ifdef WIN32
    wprintf(L"%s: %s\n", L"性别", (wchar_t *)info_buffer);
    wprintf(L"%s: %s\n", L"出生日期", (wchar_t *)birth_day);
    wprintf(L"%s: %s\n", L"住址", (wchar_t *)address);
    wprintf(L"%s: %s\n", L"公民身份号码", (wchar_t *)id_number);
    wprintf(L"%s: %s\n", L"签发机关", (wchar_t *)department);
    wprintf(L"%s: %s\n", L"证件签发日期", (wchar_t *)expire_start_day);
    wprintf(L"%s: %s\n", L"证件终止日期", (wchar_t *)expire_end_day);
    wprintf(L"%s: %s\n", L"通行证号码", (wchar_t *)pass_number);
    wprintf(L"%s: %s\n", L"签发次数", (wchar_t *)sign_count);
    wprintf(L"%s: %s\n", L"证件类型标识", (wchar_t *)type_sign);
#else
    printf("%s: %s\n", ConvertString((unsigned char *)"性别", title), info_buffer);
    printf("%s: %s\n", ConvertString((unsigned char *)"出生日期", title), birth_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"住址", title), address);
    printf("%s: %s\n", ConvertString((unsigned char *)"公民身份号码", title), id_number);
    printf("%s: %s\n", ConvertString((unsigned char *)"签发机关", title), department);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件签发日期", title), expire_start_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件终止日期", title), expire_end_day);
    printf("%s: %s\n", ConvertString((unsigned char *)"通行证号码", title), pass_number);
    printf("%s: %s\n", ConvertString((unsigned char *)"签发次数", title), sign_count);
    printf("%s: %s\n", ConvertString((unsigned char *)"证件类型标识", title), type_sign);
#endif
  }

  printf("dc_ParsePhotoInfo ... ");
  result = dc_ParsePhotoInfo(handle, 0, photo_len, photo, 0, (unsigned char *)"me.bmp");
  if (result != 0) {
    if (result == -2) {
      printf("failed to call photo decoding library!\n");
    } else {
      printf("error!\n");
    }
    return -1;
  }
  printf("ok!\n");

  return 0;
}

static int SsCardTest(HANDLE handle) {
  char card_code[33];
  char card_type[2];
  char version[5];
  char init_org_number[25];
  char card_issue_date[9];
  char card_expire_day[9];
  char card_number[10];
  char social_security_number[19];
  char name[31];
  char name_ex[21];
  char sex[2];
  char nation[3];
  char birth_place[7];
  char birth_day[9];
  unsigned char title[128], message[512];

  printf("\n-------- SS Card Test --------\n");

  printf("dc_GetSocialSecurityCardBaseInfo ... ");
  if (dc_GetSocialSecurityCardBaseInfo(handle, 1, card_code, card_type, version, init_org_number, card_issue_date, card_expire_day, card_number, social_security_number, name, name_ex, sex, nation, birth_place, birth_day) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("%s: %s\n", ConvertString((unsigned char *)"卡的识别码", title), ConvertString((unsigned char *)card_code, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"卡的类别", title), ConvertString((unsigned char *)card_type, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"规范版本", title), ConvertString((unsigned char *)version, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"初始化机构编号", title), ConvertString((unsigned char *)init_org_number, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"发卡日期", title), ConvertString((unsigned char *)card_issue_date, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"卡有效期", title), ConvertString((unsigned char *)card_expire_day, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"卡号", title), ConvertString((unsigned char *)card_number, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"社会保障号码", title), ConvertString((unsigned char *)social_security_number, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"姓名", title), ConvertString((unsigned char *)name, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"姓名扩展", title), ConvertString((unsigned char *)name_ex, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"性别", title), ConvertString((unsigned char *)sex, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"民族", title), ConvertString((unsigned char *)nation, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"出生地", title), ConvertString((unsigned char *)birth_place, message));
  printf("%s: %s\n", ConvertString((unsigned char *)"出生日期", title), ConvertString((unsigned char *)birth_day, message));

  return 0;
}

static int SsCardPinTestStep1(HANDLE handle) {
  printf("\n-------- SS Card PIN Test (Step1) --------\n");

  printf("dc_ChangeSocialSecurityCardPassword ... ");
  if (dc_ChangeSocialSecurityCardPassword(handle, 1, "123456", "111111") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  return 0;
}

static int SsCardPinTestStep2(HANDLE handle) {
  printf("\n-------- SS Card PIN Test (Step2) --------\n");

  printf("dc_ChangeSocialSecurityCardPassword ... ");
  if (dc_ChangeSocialSecurityCardPassword(handle, 1, "111111", "123456") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  return 0;
}

static int BankCardTest(HANDLE handle, int type) {
  char number[128];

  printf("\n-------- Bank Card Test --------\n");

  printf("dc_GetBankAccountNumber ... ");
  if (dc_GetBankAccountNumber(handle, type, number) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("Account Number: %s\n", number);

  return 0;
}

static int M1CardTest(HANDLE handle,QTextBrowser *tb) {
  unsigned char buffer[64];
  unsigned int len;

  printf("\n-------- M1 Card Test --------\n");

  printf("dc_reset ... ");
  if (dc_reset(handle, 10) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_config_card ... ");
  if (dc_config_card(handle, 'A') != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_card_n_hex ... ");
  if (dc_card_n_hex(handle, 0x00, &len, buffer) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");
  QString qstr;// = QString("%1").arg(num);
  tb->append("find card ok");
  qstr = QString::fromUtf8((char *)buffer);
  tb->append(qstr);
  printf("UID: %s\n", (char *)buffer);

  printf("dc_authentication_pass_hex (FFFFFFFFFFFF) ... ");
  if (dc_authentication_pass_hex(handle, 0x00, 0x00, (unsigned char *)"FFFFFFFFFFFF") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_write_hex (11223344556677889900AABBCCDDEEFF) ... ");
  if (dc_write_hex(handle, 0x01, (char *)"11223344556677889900AABBCCDDEEFF") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_read_hex ... ");
  if (dc_read_hex(handle, 0x00, (char *)buffer) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");
  tb->append("read card ok");
  qstr = QString::fromUtf8((char *)buffer);
  tb->append(qstr);
  printf("Data: %s\n", (char *)buffer);

  return 0;
}

static int ContactlessCpuCardTest(HANDLE handle, unsigned char type) {
  unsigned char buffer[2048], buffer2[2048];
  unsigned char byte_len;
  unsigned int len, len2;

  printf("\n-------- Contactless CPU Card Test (Type %c) --------\n", type);

  printf("dc_reset ... ");
  if (dc_reset(handle, 10) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_config_card ... ");
  if (dc_config_card(handle, type) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  if (type == 'A') {
    printf("dc_card_n_hex ... ");
    if (dc_card_n_hex(handle, 0x00, &len, buffer) != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");

    printf("UID: %s\n", (char *)buffer);

    printf("dc_pro_resetInt_hex ... ");
    if (dc_pro_resetInt_hex(handle, &byte_len, (char *)buffer) != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");

    printf("ATS: %s\n", (char *)buffer);
  } else {
    printf("dc_card_b_hex ... ");
    if (dc_card_b_hex(handle, (char *)buffer) != 0) {
      printf("error!\n");
      return -1;
    }
    printf("ok!\n");

    printf("Info: %s\n", (char *)buffer);
  }

  printf("dc_pro_commandlinkInt_hex (APDU Request: 0084000008) ... ");
  strcpy((char *)buffer, "0084000008");
  len = 5;
  if (dc_pro_commandlinkInt_hex(handle, len, (char *)buffer, &len2, (char *)buffer2, 7) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("APDU Response: %s\n", (char *)buffer2);

  return 0;
}

static int ContactCpuCardTest(HANDLE handle, int number) {
  unsigned char buffer[2048], buffer2[2048];
  unsigned char byte_len;
  unsigned int len, len2;

  printf("\n-------- Contact CPU Card Test (Slot %d) --------\n", number);

  printf("dc_setcpu ... ");
  if (dc_setcpu(handle, 0x0C + number) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_cpureset_hex ... ");
  if (dc_cpureset_hex(handle, &byte_len, (char *)buffer) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("ATR: %s\n", (char *)buffer);

  printf("dc_cpuapduInt_hex (APDU Request: 0084000008) ... ");
  strcpy((char *)buffer, "0084000008");
  len = 5;
  if (dc_cpuapduInt_hex(handle, len, (char *)buffer, &len2, (char *)buffer2) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("APDU Response: %s\n", (char *)buffer2);

  return 0;
}

static int Sle4442CardTest(HANDLE handle) {
  unsigned char buffer[2048];

  printf("\n-------- SLE4442 Card Test --------\n");

  printf("dc_Check_4442 ... ");
  if (dc_Check_4442(handle) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_read_4442_hex ... ");
  if (dc_read_4442_hex(handle, 0, 32, buffer) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("Data: %s\n", (char *)buffer);

  return 0;
}

static int Sle4428CardTest(HANDLE handle) {
  unsigned char buffer[2048];

  printf("\n-------- SLE4428 Card Test --------\n");

  printf("dc_Check_4428 ... ");
  if (dc_Check_4428(handle) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_read_4428_hex ... ");
  if (dc_read_4428_hex(handle, 0, 32, buffer) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("Data: %s\n", (char *)buffer);

  return 0;
}

static int EmidCardTest(HANDLE handle) {
  unsigned char buffer[2048];

  printf("\n-------- EMID Card Test --------\n");

  printf("dc_read_idcard_hex ... ");
  if (dc_read_idcard_hex(handle, 1, buffer) != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("Data: %s\n", (char *)buffer);

  return 0;
}

static int LcdDisplayTest(HANDLE handle) {
  printf("\n-------- LCD Display Test --------\n");

  printf("dc_LcdDisplay (First Line) ... ");
  if (dc_LcdDisplay(handle, 0x00, 0x00, 0x00, "1234567890 ABCDE") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_LcdDisplay (Second Line) ... ");
  if (dc_LcdDisplay(handle, 0x01, 0x00, 0x00, "FGHIJKLMNOPQRSTU") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_LcdDisplay (Third Line) ... ");
  if (dc_LcdDisplay(handle, 0x02, 0x00, 0x00, "VWXYZ abcdefghij") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  printf("dc_LcdDisplay (Fourth Line) ... ");
  if (dc_LcdDisplay(handle, 0x03, 0x00, 0x00, "klmnopqrstuvwxyz") != 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  return 0;
}

static int MultiInstanceTest(int port) {
  int result;
  HANDLE handle;
  unsigned char buffer[1024];

  printf("\n-------- Multi Instance Test --------\n");

  if (port == 100) {
    ++port;
  }

  printf("dc_init %d %d ... ", port, 115200);
  result = (int)dc_init(port, 115200);
  if (result < 0) {
    printf("error!\n");
    return -1;
  }
  printf("ok!\n");

  handle = (HANDLE)result;

  printf("dc_getver ... ");
  result = dc_getver(handle, buffer);
  if (result != 0) {
    printf("error!\n");
    dc_exit(handle);
    return -1;
  }
  printf("ok!\n");

  printf("Firmware version: %s\n", (char *)buffer);

  printf("dc_readdevsnr ... ");
  result = dc_readdevsnr(handle, buffer);
  if (result != 0) {
    printf("error!\n");
    dc_exit(handle);
    return -1;
  }
  printf("ok!\n");

  printf("Device serial number: %s\n", (char *)buffer);

  dc_exit(handle);

  return 0;
}

class AutoPause {
 public:
  AutoPause() {}
  ~AutoPause() {
    printf("\nPress ENTER key to exit ...\n");
    getchar();
  }
};
void MainWindow::on_pushButton_3_clicked()
{
    char apiver[30], input[10];
    unsigned char data[1024];
        int ret = 0, exit = 0;
        unsigned int datalen=0;
        PortParam pp;

     ZBCR_GetApiVersion(apiver);
     //set auto connect scanner
         pp.portNo = -1;
         pp.baudRate = 115200;
         //connect scanner with  callback to get decode data
         ret = ZBCR_Open(&pp,NULL,0);

         if (ZBCR_SUCCESS != ret)
         {
             //connect fail
             printf("Open fail ret = %d\n", ret);
             return ;
         }
         printf("Open scanner success\n\n");

         printf("Waiting for decode data\n");
         printf("Press q + Enter to exit\n\n");
        ZBCR_StartScan();

        ZBCR_ReadDecode(30,data,1024,&datalen);
        QString qstr;

        qstr = QString::fromUtf8((char *)data);
        ui->textBrowser->append(qstr);

         printf("Exit read decode, disconnect the scanner\n");
         ZBCR_Close();
}

void MainWindow::on_pushButton_2_clicked()
{
    int result=0;
    int port=100;
    int baud=0;
    HANDLE handle;
    unsigned char buffer[1024];
    char *p;

    printf("dc_init %d %d ... ", port, baud);
      result = (int)dc_init(port, baud);
      if (result < 0) {
        printf("error!\n");
        ui->textBrowser->setText("open fail");
        return;
      }
      printf("open device ok!\n");

      handle = (HANDLE)result;
      result=M1CardTest(handle,ui->textBrowser);
      if(result==0)
      {
          dc_beep(handle, 20);
      }


      dc_exit(handle);

}

void MainWindow::on_pushButton_clicked()
{
    int result=0;
    int port=100;
    int baud=0;
    HANDLE handle;
    unsigned char buffer[1024];
    char *p;

    printf("dc_init %d %d ... ", port, baud);
      result = (int)dc_init(port, baud);
      if (result < 0) {
        printf("error!\n");
        ui->textBrowser->setText("fail");
        return;
      }
      printf("open device ok!\n");

      handle = (HANDLE)result;

      printf("dc_getver ... ");
      result = dc_getver(handle, buffer);
      if (result != 0) {
        printf("error!\n");
        dc_exit(handle);
        return ;
      }
ui->textBrowser->setText("ok");
    //  ui.textBrowser->setText("ok");
      printf("ok!\n");

      result = IdCardTest(handle,ui->textBrowser);
      printf("id card ok!\n");
      if(result==0)
      {
          QImage *img=new QImage("me.bmp");
          ui->label->setPixmap(QPixmap::fromImage(*img));
      }
      dc_beep(handle, 20);

      dc_exit(handle);


}

void MainWindow::on_pushButton_4_clicked()
{
    int result=0;
    int port=100;
    int baud=0;
    HANDLE handle;
    char buffer[1024];
    char *p;

    printf("dc_init %d %d ... ", port, baud);
      result = (int)dc_init(port, baud);
      if (result < 0) {
        printf("error!\n");
        ui->textBrowser->setText("open device fail");
        return;
      }
      printf("open device ok!\n");

      if (dc_GetBankAccountNumber(result, 0, buffer) != 0) {
        printf("error!\n");
      ui->textBrowser->setText("请放置有闪付功能的银行卡进行测试");
        return;
      }
      printf("ok!\n");
      QString qstr;// = QString("%1").arg(num);
      ui->textBrowser->append("read bank card ok");
      qstr = QString::fromUtf8((char *)buffer);
      ui->textBrowser->append(qstr);
      printf("Account Number: %s\n", buffer);

      dc_beep(handle, 20);

      dc_exit(handle);
}
